# Script R pour tester les bugs sur l'extension DESS (à priori pour QSS aussi...)
# Comparaison des schema numeriques Euler et RK4 selon les différentes implémentation comparées (R vs DE vs DESS)


rm(list=ls()); #nettoyage de l'espace de travail

timeStep <- 0.001 # pas de temps d'intégration du schema numerique (a prendre similaire a celui des vpz pour DE et DESS)
tt <- seq(0,15-timeStep, by=timeStep) # vecteur des différents pas de temps correspondant à cette discretisation
nstep <- length(tt)-1 # nombre de pas de calcul correspondant (1er = init, pas de calcul)

# valeurs initiales des variables d'etats du modele (a prendre similaire a celui des vpz pour DE et DESS)
x0 <- 10
y0 <- 5
# valeurs initiales des parametres du modele (a prendre similaire a celui des vpz pour DE et DESS)
alpha <- 1.5
beta <- 1
delta <- 3
gamma <- 1





library('rvle'); # chargement de la librairie R de VLE : rvle 

# definition d'une fonction pour executer un nombre indeterminé de vpz d'un pkg donné
# ... arguments optionnels de la fonction run de rvle
runAllVzp <- function(VPZ, PKGS="LotkaVolterra", proc="dual", ...) {
  Exp <- list(); # initialisation des objets temporaires de type list que l'on va remplir
  res <- list();

  for (i in 1:length(VPZ)) { # boucle sur les différents fichiers vpz demandés 
    Exp[[i]] <- new("Rvle", pkg=PKGS, file=VPZ[i]); # chargement du simulateur
    show(Exp[[i]]); # affichage des données contenues dans le simulateur
    config(Exp[[i]])$proc <- proc; # utilisation des différents processeurs
    config(Exp[[i]]); # affichage des données de configuration du simulateur
    Exp[[i]] <- run(Exp[[i]], ...); # execution de la simulation
    res[[i]] <- results(Exp[[i]])[[1]]; # extraction des resultats de simulations (venant des vues de type storage) 1 seule dans notre cas
    str(res[[i]]); # affichage de la structure des données de sorties
    if (i==1) { # cas particulier pour le 1er vpz traité étant donné que l'on procede par concatenation de listes (Exp & res)
      resMat <- res[[i]][, 1:3]; # matrice des resultats (on prend ici la 1ere colonne qui est celle des temps)
      residus <- rep(0, length(res[[i]][, 2])); # matrice des residus relatifs (relatifs à la 1ere simulation, donc 0 pour la 1ere)
      simName <- c(paste(Exp[[i]]@file, ' - x'), paste(Exp[[i]]@file, ' - y')); # extraction des noms des simulations
    } else {
      resMat <- cbind(resMat, res[[i]][, 2:3]); # matrice des resultats (on ne prend pas ici la 1ere colonne qui est celle des temps)
      residus <- cbind(residus, (res[[i]][, 2:3] - res[[1]][, 2:3]) / res[[1]][, 2:3]); # matrice des residus relatifs (relatifs à la 1ere simulation)
      simName <- c(simName, c(paste(Exp[[i]]@file, ' - x'), paste(Exp[[i]]@file, ' - y'))); # extraction des noms des simulations
    } 
  }
  return(list(Exp=Exp, Sim=resMat, Res=residus, Name=simName)) # l'objet renvoyé par la fonction (liste de 4 éléments)
}

plotSimTime <- function(XX, sub="", ...) {
  matplot(XX$Sim[, 1], XX$Sim[, -1],
          main=c('Resultats des simulations', sub), xlab="time", ylab="Effectifs : x, y",
          col=1:(2*length(XX$Exp)), type="l", lty=2, ...); # graphiques des trajectoires des variables d'état en fonction du temps'
  legend("topleft", legend=XX$Name, col=1:length(XX$Name), lty=2, ncol=length(XX$Exp), cex=0.5); #écriture de la legende du graph  
}

plotResTime <- function(XX, ...) {
  matplot(XX$Sim[, 1], XX$Res, 
          col=3:(2*length(XX$Exp)), type="l", lty=2, 
          main=c("residus relatif", paste("max%:", round(max(XX$Res, na.rm=TRUE)*100, 4),  " - min%:", round(min(XX$Res, na.rm=TRUE)*100, 4))),
          xlab="time", ylab="residus relatif", ...); # graphique des residus relatifs en fonction du temps
  legend("topleft", legend=XX$Name[-(1:2)], col=3:length(XX$Name), lty=2, ncol=(length(XX$Exp)-1), cex=0.5); #écriture de la legende du graph
}

plotSimComparison <- function(XX, sub="", ...) {
  par(mfrow=c(2,1)) # decoupage de la fenetre graphique en 2
  plotSimTime(XX, sub=sub, ...);
  plotResTime(XX, ...);
}

plotSimTrajectory <- function(XX, sub="", ...) {
  matplot(XX$Sim[,seq(2,length(XX$Sim[1,]), by=2)], 
          XX$Sim[,seq(3,length(XX$Sim[1,]), by=2)], 
          main=c('Resultats des simulations', sub), xlab="Effectifs x", ylab="Effectifs y", 
          col=1:length(XX$Exp), type="l", lty=2, ...)
  legend("topright", legend=XX$Name[seq(2,length(XX$Sim[1,]), by=2)], col=1:length(XX$Exp), lty=2, cex=0.5);
}

cXX <- function(x, y) {
  list(Exp=c(x$Exp, y$Exp), 
       Sim=cbind(x$Sim, y$Sim[,-1]), 
       Res = cbind(x$Res, (y$Sim[,-1] - rep(x$Sim[, 2:3], length(y$Sim[,-1])/2)) 
                   / rep(x$Sim[, 2:3], length(y$Sim[,-1])/2)), #recalcul des résidus relatifs par rapport à ceux de la 1ere simulation du premier objet à concatener au
       Name=c(x$Name, y$Name))
}


compareToExact <- function(XX) {
  L <- function(X, Y) {
    return(beta*Y + gamma*X - alpha*log(Y) - delta*log(X));
  }
  L0 <- L(x0, y0); #L = cst = L0
    
  x <- seq(0, 15, by=0.01)
  y <- seq(0, 15, by=0.01)
  z <- outer(x[-1], y[-1], L)
  
  par(mfrow=c(1,2))
  image(x[-1], y[-1], z, main=c(paste('L0=', L0, sep=""), "+-10%"), breaks=c(min(z), L0 * c(0.9, 1.1), max(z)), col=c("red", "green", "red"))
  contour(x[-1], y[-1], z, add=TRUE, levels=L0 * c(0.9, 1, 1.1), lwd=2, drawlabels=FALSE) 
  lines(XX$Sim[c(2, 3)], lwd=1.5, col="#0000FF80", type="o")
  
  plot(X <- L(XX$Sim[2], XX$Sim[3])[,1], main=c( "max(L-L0)/L0 = ", max(abs(X-L0))/L0))
  abline(h=L0, col='red')
}





#R_Euler

x <- rep(NA, nstep)
y <- rep(NA, nstep)
x[1] <- x0
y[1] <- y0
for (i in (1:(nstep-1)+1)) { #boucle d'avancement du pas de temps
    x[i] <- x[i-1] + x[i-1] * (alpha - beta * y[i-1]) * timeStep; 
    y[i] <- y[i-1] - y[i-1] * (delta - gamma * x[i-1]) * timeStep;
}

# construction d'un objet de format similaire à celui renvoyé par la fonction runAllVpz
obs <- c(TRUE, rep(FALSE, 49))
R_euler <- list(Exp="", Sim=data.frame(time=tt[obs], x=x[obs], y=y[obs]), Res=rep(0, length(tt[obs])), Name=c("R_euler - x", "R_euler - y"))
X11();
par(mfrow=c(1,2))
matplot(R_euler$Sim[, 1], R_euler$Sim[, -1], cex=0.1, typ="o")
plot(R_euler$Sim[, -1], cex=0.1, typ="o")


#Runge Kutta


x <- rep(NA, nstep)
y <- rep(NA, nstep)
x[1] <- x0
y[1] <- y0
for (i in (1:(nstep-1)+1)) {
    k11 = x[i-1] * (alpha - beta * y[i-1]);
    k12 = y[i-1] * (-delta + gamma * x[i-1]);
    k21 = alpha * (x[i-1] + timeStep*k11/2) - beta * (x[i-1] + timeStep*k11/2)*(y[i-1] + timeStep*k12/2);
    k22 = -delta * (y[i-1] + timeStep*k12/2) + gamma * (x[i-1] + timeStep*k11/2) * (y[i-1] + timeStep*k12/2);
    k31 = alpha * (x[i-1] + timeStep*k21/2) - beta * (x[i-1] + timeStep*k21/2) * (y[i-1] + timeStep*k22/2);
    k32 = -delta * (y[i-1] + timeStep*k22/2) + gamma * (x[i-1] + timeStep*k21/2) * (y[i-1] + timeStep*k22/2);
    k41 = alpha * (x[i-1] + timeStep*k31) - beta * (x[i-1] + timeStep*k31) * (y[i-1] + timeStep*k32);
    k42 = -delta * (y[i-1] + timeStep*k32) + gamma * (x[i-1] + timeStep*k31) * (y[i-1] + timeStep*k32);
    x[i] = x[i-1] + timeStep * (k11 + 2*k21 + 2*k31 + k41) / 6;
    y[i] = y[i-1] + timeStep * (k12 + 2*k22 + 2*k32 + k42) / 6;
}

obs <- c(TRUE, rep(FALSE, 49))
R_rk4 <- list(Exp="", Sim=data.frame(time=tt[obs], x=x[obs], y=y[obs]), Res=rep(0, length(tt[obs])), Name=c("R_rk4r - x", "R_rk4 - y"))
X11();
par(mfrow=c(1,2))
matplot(R_rk4$Sim[, 1], R_rk4$Sim[, -1], cex=0.1, typ="o")
plot(R_rk4$Sim[, -1], cex=0.1, typ="o")




#Solution Exacte

L <- function(X, Y) {
  return(beta*Y + gamma*X - alpha*log(Y) - delta*log(X))
}
L0 <- L(x0, y0) #L = cst = L0

X11();
par(mfrow=c(2,2))
plot(L(R_euler$Sim[2], R_euler$Sim[3])[,1])
abline(h=L0, col='red')
plot(L(R_rk4$Sim[2], R_rk4$Sim[3])[,1])
abline(h=L0, col='red')
matplot(cbind(L(R_euler$Sim[2], R_euler$Sim[3])[,1], L(R_rk4$Sim[2], R_rk4$Sim[3])[,1]), type="l")
boxplot(cbind(L(R_euler$Sim[2], R_euler$Sim[3])[,1], L(R_rk4$Sim[2], R_rk4$Sim[3])[,1]))


x <- seq(0, 15, by=0.01)
y <- seq(0, 15, by=0.01)
z <- outer(x[-1], y[-1], L)
X11();
par(mfrow=c(1,1))
image(x[-1], y[-1], z, main=paste('L0=', L0, sep=""), breaks=c(min(z), L0 * c(0.9, 1.1), max(z)), col=c("red", "green", "red"))
contour(x[-1], y[-1], z, add=TRUE, levels=L0 * c(0.9, 1, 1.1), lwd=2, drawlabels=FALSE)




X_dess_rk4 <- runAllVzp(c('LV_DESS_rk4.vpz'))
X_dess_euler <- runAllVzp(c('LV_DESS_euler.vpz'))
X_de_rk4 <- runAllVzp(c('LV_DE_rk4.vpz'))
X_de_euler <- runAllVzp(c('LV_DE_euler.vpz'))

X_dess <- cXX(X_dess_rk4, X_dess_euler)
X_de <- cXX(X_de_rk4, X_de_euler)
X_euler <- cXX(cXX(R_euler, X_dess_euler), X_de_euler)
X_rk4 <- cXX(cXX(R_rk4, X_dess_rk4), X_de_rk4)


X11();
plotSimComparison(X_euler, sub="Euler")
X11();
plotSimComparison(cXX(R_euler, X_dess_euler), sub="Euler - R vs DESS")
X11();
plotSimComparison(cXX(R_euler, X_de_euler), sub="Euler - R vs DE")

X11();
plotSimComparison(X_rk4, sub="RK4")
X11();
plotSimComparison(cXX(R_rk4, X_dess_rk4), sub="RK4 - R vs DESS")
X11();
plotSimComparison(cXX(R_rk4, X_de_rk4), sub="RK4 - R vs DE")





X_qss_simple <- runAllVzp(c('LV_QSSsimple.vpz'))
X_qss_multiple <- runAllVzp(c('LV_QSSmultiple.vpz'))

X11();
par(mfrow=c(3,2))
plot(X <- L(X_dess_euler$Sim[2], X_dess_euler$Sim[3])[,1], main=c("DESS Euler", max(abs(X-L0))/L0))
abline(h=L0, col='red')
plot(X <-L(X_de_euler$Sim[2], X_de_euler$Sim[3])[,1], main=c("DE Euler", max(abs(X-L0))/L0))
abline(h=L0, col='red')
plot(X <-L(X_dess_rk4$Sim[2], X_dess_rk4$Sim[3])[,1], main=c("DESS RK4", max(abs(X-L0))/L0))
abline(h=L0, col='red')
plot(X <-L(X_de_rk4$Sim[2], X_de_rk4$Sim[3])[,1], main=c("DE RK4", max(abs(X-L0))/L0))
abline(h=L0, col='red')
plot(X <-L(X_qss_simple$Sim[2], X_qss_simple$Sim[3])[,1], main=c("QSS simple", max(abs(X-L0))/L0))
abline(h=L0, col='red')
plot(X <-L(X_qss_multiple$Sim[2], X_qss_multiple$Sim[3])[,1], main=c("QSS multiple", max(abs(X-L0))/L0))
abline(h=L0, col='red')





X11();
compareToExact(X_qss_simple)



require(rgl)
zlim <- range(z, na.rm = TRUE)
zlen <- 100
colorlut <- heat.colors(zlen) # height color lookup table
col <- colorlut[(zlen-1)/(zlim[2]-zlim[1])*(z-zlim[1])+1]

plot3d(x[-1], y[-1], z, col=col, main=paste('L0 =', L0), xlab="x", ylab="y", zlab="L", type='n')
surface3d(x[-1], y[-1], z, color=col, back="lines")

