# Script R du paquet LotkaVolterra tutoriel sur les Equations Différentielles dans RECORD

rm(list=ls()); # nettoyage de l'espace de travail
library('rvle'); # chargement de la librairie R de VLE : rvle 

# definition d'une fonction pour executer un nombre indeterminé de vpz d'un pkg donné
# ... arguments optionnels de la fonction run de rvle
runAllVzp <- function(VPZ, PKGS="LotkaVolterra", proc="dual", ...) {
  Exp <- list(); # initialisation des objets temporaires de type list que l'on va remplir
  res <- list();

  for (i in 1:length(VPZ)) { # boucle sur les différents fichiers vpz demandés 
    Exp[[i]] <- new("Rvle", pkg=PKGS, file=VPZ[i]); # chargement du simulateur
    show(Exp[[i]]); # affichage des données contenues dans le simulateur
    config(Exp[[i]])$proc <- proc; # utilisation des différents processeurs
    config(Exp[[i]]); # affichage des données de configuration du simulateur
    Exp[[i]] <- run(Exp[[i]], ...); # execution de la simulation
    res[[i]] <- results(Exp[[i]])[[1]]; # extraction des resultats de simulations (venant des vues de type storage) 1 seule dans notre cas
    str(res[[i]]); # affichage de la structure des données de sorties
    if (i==1) { # cas particulier pour le 1er vpz traité étant donné que l'on procede par concatenation de listes (Exp & res)
      resMat <- res[[i]][, 1:3]; # matrice des resultats (on prend ici la 1ere colonne qui est celle des temps)
      residus <- rep(0, length(res[[i]][, 2])); # matrice des residus relatifs (relatifs à la 1ere simulation, donc 0 pour la 1ere)
      simName <- c(paste(Exp[[i]]@file, ' - x'), paste(Exp[[i]]@file, ' - y')); # extraction des noms des simulations
    } else {
      resMat <- cbind(resMat, res[[i]][, 2:3]); # matrice des resultats (on ne prend pas ici la 1ere colonne qui est celle des temps)
      residus <- cbind(residus, (res[[i]][, 2:3] - res[[1]][, 2:3]) / res[[1]][, 2:3]); # matrice des residus relatifs (relatifs à la 1ere simulation)
      simName <- c(simName, c(paste(Exp[[i]]@file, ' - x'), paste(Exp[[i]]@file, ' - y'))); # extraction des noms des simulations
    } 
  }
  return(list(Exp=Exp, Sim=resMat, Res=residus, Name=simName)) # l'objet renvoyé par la fonction (liste de 4 éléments)
}

#fonctions graphiques
#prend en entrée un objet ayant la même structure que celle généré par la fonction runAllVpz
# ... arguments de la fonction matplot
plotSimTime <- function(XX, sub="", ...) {
  matplot(XX$Sim[, 1], XX$Sim[, -1],
          main=c('Resultats des simulations', sub), xlab="time", ylab="Effectifs : x, y",
          col=1:(2*length(XX$Exp)), type="l", lty=2, ...); # graphiques des trajectoires des variables d'état en fonction du temps'
  legend("topleft", legend=XX$Name, col=1:length(XX$Name), lty=2, ncol=length(XX$Exp), cex=0.5); #écriture de la legende du graph  
}

plotResTime <- function(XX, ...) {
  matplot(XX$Sim[, 1], XX$Res, 
          col=3:(2*length(XX$Exp)), type="l", lty=2, 
          main=c("residus relatif", paste("max%:", round(max(XX$Res, na.rm=TRUE)*100, 4),  " - min%:", round(min(XX$Res, na.rm=TRUE)*100, 4))),
          xlab="time", ylab="residus relatif", ...); # graphique des residus relatifs en fonction du temps
  legend("topleft", legend=XX$Name[-(1:2)], col=3:length(XX$Name), lty=2, ncol=(length(XX$Exp)-1), cex=0.5); #écriture de la legende du graph
}

plotSimComparison <- function(XX, sub="", ...) {
  layout(matrix(1:2)) # decoupage de la fenetre graphique en 2
  plotSimTime(XX, sub=sub, ...);
  plotResTime(XX, ...);
}

plotSimTrajectory <- function(XX, sub="", ...) {
  matplot(XX$Sim[,seq(2,length(XX$Sim[1,]), by=2)], 
          XX$Sim[,seq(3,length(XX$Sim[1,]), by=2)], 
          main=c('Resultats des simulations', sub), xlab="Effectifs x", ylab="Effectifs y", 
          col=1:length(XX$Exp), type="l", lty=2, ...)
  legend("topright", legend=XX$Name[seq(2,length(XX$Sim[1,]), by=2)], col=1:length(XX$Exp), lty=2, cex=0.5);
}
  
# definition d'une fonction pour concatener deux objets du type renvoyé par la fonction runAllVpz
cXX <- function(x, y) {
  list(Exp=c(x$Exp, y$Exp), 
       Sim=cbind(x$Sim, y$Sim[,-1]), 
       Res = cbind(x$Res, (y$Sim[,-1] - rep(x$Sim[, 2:3], length(y$Sim[,-1])/2)) / rep(x$Sim[, 2:3], length(y$Sim[,-1])/2)), #recalcul des résidus relatifs par rapport à ceux de la 1ere simulation du premier objet à concatener au
       Name=c(x$Name, y$Name))
}


  
  
  
# execution des simulation et génération des graphiques associés
log <- FALSE

X_dess_rk4 <- runAllVzp(c('LV_DESS_rk4.vpz'), condLogDbg.log=log)
X_dess_euler <- runAllVzp(c('LV_DESS_euler.vpz'), condLogDbg.log=log)
X_qss_simple <- runAllVzp(c('LV_QSSsimple.vpz'), condLogDbg.log=log)
X_qss_multiple <- runAllVzp(c('LV_QSSmultiple.vpz'), condLogDbg.log=log)
X_de_rk4 <- runAllVzp(c('LV_DE_rk4.vpz'), condLogDbg.log=log)
X_de_euler <- runAllVzp(c('LV_DE_euler.vpz'), condLogDbg.log=log)

plotSimComparison(X_dess <- cXX(X_dess_rk4, X_dess_euler), sub="DESS - rk4 vs euler")
plotSimComparison(X_qss <- cXX(X_qss_simple, X_qss_multiple), sub="QSS - simple vs multiple")
plotSimComparison(X_de <- cXX(X_de_rk4, X_de_euler), sub="DE - RK4 vs euler")
plotSimComparison(X_euler <- cXX(X_dess_euler, X_de_euler), sub="Euler - DESS vs DE")
plotSimComparison(X_rk4 <- cXX(X_dess_rk4, X_de_rk4), sub="RK4 - DESS vs DE")
plotSimComparison(X_dessqss <- cXX(X_qss, X_dess), sub="QSS vs DESS")

plotSimComparison(cXX(X_dess_rk4, X_qss_simple), sub="DESS rk4 vs QSS::Simple")


plotSimTrajectory(X_all <- cXX(cXX(X_qss, X_dess), X_de), sub="All")


X_dess_perturb_x <- runAllVzp(c('perturb/LV_DESS_perturb_x.vpz'), condFSAperturb.dayPerturb=9, condLogDbg.log=log)
X_qss_simple_perturb_x <- runAllVzp(c('perturb/LV_QSSsimple_perturb_x.vpz'), condFSAperturb.dayPerturb=9, condLogDbg.log=log)
X_qss_multiple_perturb_x <- runAllVzp(c('perturb/LV_QSSmultiple_perturb_x.vpz'), condFSAperturb.dayPerturb=9, condLogDbg.log=log)
X_dess_perturb_y <- runAllVzp(c('perturb/LV_DESS_perturb_y.vpz'), condLogDbg.log=log)
X_qss_simple_perturb_y <- runAllVzp(c('perturb/LV_QSSsimple_perturb_y.vpz'), condLogDbg.log=log)
X_qss_multiple_perturb_y <- runAllVzp(c('perturb/LV_QSSmultiple_perturb_y.vpz'), condLogDbg.log=log)

plotSimComparison(X_perturb_x <- cXX(cXX(X_dess_perturb_x, X_qss_simple_perturb_x), X_qss_multiple_perturb_x), sub="X perturb")
plotSimComparison(X_perturb_y <- cXX(cXX(X_dess_perturb_y, X_qss_simple_perturb_y), X_qss_multiple_perturb_y), sub="Y perturb")

plotSimTrajectory(X_perturb_x, sub="X perturb")
plotSimTrajectory(X_perturb_y, sub="Y perturb")

plotSimTrajectory(Xstationnaire <- runAllVzp(c('LV_DESS_rk4.vpz', 'LV_DESS_euler.vpz', 'LV_QSSsimple.vpz', 'LV_QSSmultiple.vpz'),
                                             condLVx_param.alpha=c(5), condLVy_param.delta=c(10)), sub="Stationnaire")

